#include "ImageRenderer.h"
#include <GLUT/glut.h>
#include <math.h>
#include <algorithm>
#include <iostream>


static void float2rgb(float value,float *col)	//simple color-coding routine
{
   static const float dx = 0.8f;
   
   if (value<0.0f) value=0.0f;
   if (value>1.0f) value=1.0f;
   
   value = (6.0f-2.0f*dx)*value+dx;
   col[0] = std::max(0.0f,(3.0f-fabsf(value-4.0f)-fabsf(value-5.0f))/2.0f);
   col[1] = std::max(0.0f,(4.0f-fabsf(value-2.0f)-fabsf(value-4.0f))/2.0f);
   col[2] = std::max(0.0f,(3.0f-fabsf(value-1.0f)-fabsf(value-2.0f))/2.0f);
}



template<class T> ImageRenderer<T>::ImageRenderer(): draw_colored(false) 
{ }



template <class T> void ImageRenderer<T>::draw(Image<T>& image)
{
   glClearColor(1,1,1,1);
   glClear(GL_COLOR_BUFFER_BIT);
   
   int dimX = image.getWidth();
   int dimY = image.getHeight();
   
   static unsigned char buf[2000*2000*3];
   int j=0;

   T m,M; 
   if (draw_colored)
      image.range(m,M);
   
   for(int i=dimY-1;i>=0;i--)
     for(const T *s=image[i], *e=s+dimX;s<e;s++)
     {
       float col[3];
	   
	   T v = *s;	   
	   if (draw_colored)														//Interpret the image as a scalar field, 
	   {																		//and visualize it by color mapping. For this,
	     float val = (float(v)-float(m))/(float(M)-float(m));					//we compute first the scalar values, normalized in [0,1]
																				//Next, we color map these using our current colormap.
																				//Finally, we map these to colors in [0,255]^3 (bytes).
		 float2rgb(val,col);
		 col[0] *= 255;
		 col[1] *= 255;
		 col[2] *= 255;
	   }	 
       else
		 col[0]=col[1]=col[2]=v;
		   
       buf[j++] = (unsigned char)(int)(col[0]);
       buf[j++] = (unsigned char)(int)(col[1]);
       buf[j++] = (unsigned char)(int)(col[2]);
     }


   GLint viewport[4];	
   glGetIntegerv(GL_VIEWPORT,viewport);
   
   float scale = std::min(float(viewport[2])/dimX,float(viewport[3])/dimY);
   float ox    = std::max((viewport[2]-scale*dimX)/2,0.0f);
   float oy    = std::max((viewport[3]-scale*dimY)/2,0.0f);
   
   glPixelStorei(GL_UNPACK_ALIGNMENT,1);	
   glPixelZoom(scale,scale);
   glRasterPos2f(ox,oy);
   glDrawPixels(dimX,dimY,GL_RGB,GL_UNSIGNED_BYTE,buf);
   
   glutSwapBuffers();
}


//----   Specializations  --------------------------------------------------------


template<> ImageRenderer<RGBPixel>::ImageRenderer(): draw_colored(true) 
{ }



template <> void ImageRenderer<RGBPixel>::draw(Image<RGBPixel>& image)
{
   glClearColor(1,1,1,1);
   glClear(GL_COLOR_BUFFER_BIT);
   
   int dimX = image.getWidth();
   int dimY = image.getHeight();
   
   static unsigned char buf[2000*2000*3];
   
   int j=0;
   for(int i=dimY-1;i>=0;i--)
     for(const RGBPixel *s=image[i], *e=s+dimX;s<e;s++)
     {
       float col[3];
	   if (draw_colored)
	   {
	      col[0] = s->r;
	      col[1] = s->g;
	      col[2] = s->b;
	   }
	   else col[0]=col[1]=col[2] = (s->r+s->g+s->b)/3;	  
	   
       buf[j++] = (unsigned char)(int)(col[0]);
       buf[j++] = (unsigned char)(int)(col[1]);
       buf[j++] = (unsigned char)(int)(col[2]);
     }


   GLint viewport[4];	
   glGetIntegerv(GL_VIEWPORT,viewport);
   
   float scale = std::min(float(viewport[2])/dimX,float(viewport[3])/dimY);
   float ox    = std::max((viewport[2]-scale*dimX)/2,0.0f);
   float oy    = std::max((viewport[3]-scale*dimY)/2,0.0f);
   
   glPixelStorei(GL_UNPACK_ALIGNMENT,1);	
   glPixelZoom(scale,scale);
   glRasterPos2f(ox,oy);
   glDrawPixels(dimX,dimY,GL_RGB,GL_UNSIGNED_BYTE,buf);
   
   glutSwapBuffers();
}




template class ImageRenderer< char >;
template class ImageRenderer< byte >;
template class ImageRenderer< short >;
template class ImageRenderer< int >;
template class ImageRenderer< float >;
template class ImageRenderer< double >;
template class ImageRenderer< RGBPixel >;



