#include <string.h>
#include <iostream>
#include <vector>
#include <GLUT/glut.h>
#include "Image.h"
#include "ImageRenderer.h"
#include "FMM.h"
#include "Flags.h"


using namespace std;




Image<float>*		  input_image		 =0;				//Input image (grayscale)
Image<float>*		  thresholded_image  =0;				//Thresholding of input image
Image<float>*		  distance_transform =0;				//Distance-transform of boundary of thresholded image
ImageRenderer<float>* renderer			 =0;				//Image renderer
float				  k = -100;								//Grayscale threshold (see FLAGS class)

vector<Image<float>*> images;								//Set of all images (with descriptive names) to display
vector<const char*>   image_names;
int					  f_display = 0;						//Current image to display





void computeDT(Image<float>& input_image,Image<float>& thresholded_image,Image<float>& distance_transform,float k)
{															//Threshold the input image with value k, to generate a binary image.
															//Next, compute the distance-transform of the boundary of this 
   thresholded_image = input_image;							//binary image, using the Fast Marching Method.

   FLAGS flags(thresholded_image,k); 	

   distance_transform = thresholded_image;
      
   FastMarchingMethod fmm(&distance_transform,&flags);		//Create fast marching method engine
   int iterations = fmm.execute();							//Compute the distance transform
   cout<<"FMM iterations: "<<iterations<<endl;				//Display how many iterations we needed
}





void viewing(int width, int height)							//GLUT window-reshape callback
{
	glutReshapeWindow(width,height);						//Initialze the OpenGL modelview and projection
	glMatrixMode(GL_MODELVIEW);								//matrices so as to have a 1-to-1 correspondence
	glLoadIdentity();										//between world coordinates and pixels
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0,width,0,height);
	glViewport(0,0,width,height);
}



void keyboard(unsigned char ch,int,int)						//GLUt keyboard callback
{
    switch(ch)
    {
      case ' ':												//Cycle through the available images
	  { 
	    f_display = (++f_display)%(images.size()); 
		cout<<"Showing: "<<image_names[f_display]<<endl;
		break; 
	  }
	  case '+':												//Increase threshold to generate new images
	  case '=':
	  {
		k++;		
	    computeDT(*input_image,*thresholded_image,*distance_transform,k);
		break;
	  }	
	  case '-':												//Decrease threshold to generate new images
	  case '_':
	  {
		k--;		
	    computeDT(*input_image,*thresholded_image,*distance_transform,k);
		break;
	  }	
      case 'c':												//Display images either as grayscale or color-mapped
	  case 'C':
	    renderer->setDrawColored(!renderer->getDrawColored()); 
		break;
    }

    glutPostRedisplay();
}


void display()												//GLUT display callback
{
  renderer->draw(*images[f_display]);						//Display currently selected image
}




int main(int argc,char* argv[])
{
   cout<<"2. Fast Marching Method / Distance transforms"<<endl;
   cout<<"In the graphics window, press"<<endl;
   cout<<"   space: cycle through the images to display"<<endl;
   cout<<"   +,-:   increase/decrease image threshold"<<endl;
   cout<<"   c,C:   show images as grayscale or color-mapped"<<endl;
   
   
   glutInit(&argc,argv);
   const char* filename = (argc>1)? argv[1] : "DATA/skinlesion.ppm";
   
   if (strstr(filename,".pgm"))	
      input_image = Image<float>::readPGM(filename);				//1. Read an input image
   else if (strstr(filename,".ppm"))								//   If color, convert to luminance
      input_image = Image<float>::readPPM(filename);
	  	  
   if (!input_image) 
   { cout<<"Can not open file: "<<filename<<endl; return 1; }

   thresholded_image  = new Image<float>(*input_image);				//2. Allocate thresholded image
   distance_transform = new Image<float>(*input_image);				//   and distance-transform image

   computeDT(*input_image,*thresholded_image,*distance_transform,k);//3. Do the thresholding and compute
																	//   the distance-transform

   glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
   glutInitWindowSize(input_image->getWidth(),input_image->getHeight());
   glutCreateWindow("2. Fast Marching Method / Distance transforms");
   
																	//4. Gather all images to visualize
   images.push_back(input_image);		 image_names.push_back("Input image");
   images.push_back(thresholded_image);  image_names.push_back("Thresholded image");
   images.push_back(distance_transform); image_names.push_back("Distance transform");
   
   renderer = new ImageRenderer<float>();							//5. Create an image renderer

   glutKeyboardFunc(keyboard);
   glutDisplayFunc(display);
   glutReshapeFunc(viewing);
   glutMainLoop();
   
   return 0;
}


